@@ -36,6 +36,7 @@
 #include <dbus/dbus-glib.h>
 
 #include <NetworkManager.h>
+#include "nm-glib-compat.h"
 #include <libnm-util/nm-connection.h>
 #include <libnm-util/nm-setting-ip4-config.h>
 #include <libnm-util/nm-setting-connection.h>
@@ -236,7 +237,10 @@ add_one_option_to_envp (gpointer key, gpointer value, gpointer user_data)
 }
 
 static char **
-construct_envp (const char *uuid, NMIP4Config *ip4_config, NMDHCP4Config *dhcp4_config)
+construct_envp (const char *uuid,
+                const char *id,
+                NMIP4Config *ip4_config,
+                NMDHCP4Config *dhcp4_config)
 {
 	guint32 env_size = 0;
 	char **envp;
@@ -263,7 +267,8 @@ construct_envp (const char *uuid, NMIP4Config *ip4_config, NMDHCP4Config *dhcp4_
 	           + 1 /* hostname */
 	           + g_slist_length (routes)
 	           + 1 /* routes length */
-	           + 1 /* connection UUID */;
+	           + 1 /* connection UUID */
+	           + 1 /* connection ID */;
 
 	if (dhcp4_config) {
 		options = nm_dhcp4_config_get_options (dhcp4_config);
@@ -274,6 +279,8 @@ construct_envp (const char *uuid, NMIP4Config *ip4_config, NMDHCP4Config *dhcp4_
 
 	if (uuid)
 		envp[envp_idx++] = g_strdup_printf ("CONNECTION_UUID=%s", uuid);
+	if (id)
+		envp[envp_idx++] = g_strdup_printf ("CONNECTION_ID=%s", id);
 
 	/* IP4 config stuff */
 	for (iter = addresses, num = 0; iter; iter = g_slist_next (iter)) {
@@ -383,6 +390,7 @@ dispatch_scripts (const char *action,
                   const char *parent_iface,
                   NMDeviceType type,
                   const char *uuid,
+                  const char *id,
                   NMIP4Config *ip4_config,
                   NMDHCP4Config *dhcp4_config)
 {
@@ -428,7 +436,7 @@ dispatch_scripts (const char *action,
 	}
 	g_dir_close (dir);
 
-	envp = construct_envp (uuid, ip4_config, dhcp4_config);
+	envp = construct_envp (uuid, id, ip4_config, dhcp4_config);
 
 	for (iter = scripts; iter; iter = g_slist_next (iter)) {
 		gchar *argv[4];
@@ -474,7 +482,7 @@ nm_dispatcher_action (Handler *h,
 	Dispatcher *d = g_object_get_data (G_OBJECT (h), "dispatcher");
 	NMConnection *connection;
 	char *iface = NULL, *parent_iface = NULL;
-	const char *uuid = NULL;
+	const char *uuid = NULL, *id = NULL;
 	NMDeviceType type = NM_DEVICE_TYPE_UNKNOWN;
 	NMDeviceState dev_state = NM_DEVICE_STATE_UNKNOWN;
 	NMDevice *device = NULL;
@@ -486,15 +494,16 @@ nm_dispatcher_action (Handler *h,
 	if (d->quit_timeout)
 		g_source_remove (d->quit_timeout);
 	if (!d->persist)
-		d->quit_timeout = g_timeout_add (10000, quit_timeout_cb, NULL);
+		d->quit_timeout = g_timeout_add_seconds (10, quit_timeout_cb, NULL);
+
+	/* Hostname changes don't require a device nor contain a connection */
+	if (!strcmp (action, "hostname"))
+		goto dispatch;
 
 	connection = nm_connection_new_from_hash (connection_hash, error);
 	if (connection) {
-		NMSettingConnection *s_con;
-
-		s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
-		if (s_con)
-			uuid = nm_setting_connection_get_uuid (s_con);
+		uuid = nm_connection_get_uuid (connection);
+		id = nm_connection_get_id (connection);
 	} else {
 		g_warning ("%s: Invalid connection: '%s' / '%s' invalid: %d",
 		           __func__,
@@ -505,10 +514,6 @@ nm_dispatcher_action (Handler *h,
 		*error = NULL;
 	}
 
-	/* Hostname changes don't require a device */
-	if (!strcmp (action, "hostname"))
-		goto dispatch;
-
 	/* interface name */
 	value = g_hash_table_lookup (device_props, NMD_DEVICE_PROPS_INTERFACE);
 	if (!value || !G_VALUE_HOLDS_STRING (value)) {
@@ -559,7 +564,7 @@ nm_dispatcher_action (Handler *h,
 	}
 
 dispatch:
-	dispatch_scripts (action, iface, parent_iface, type, uuid, ip4_config, dhcp4_config);
+	dispatch_scripts (action, iface, parent_iface, type, uuid, id, ip4_config, dhcp4_config);
 
 	if (device)
 		g_object_unref (device);
@@ -750,6 +755,7 @@ main (int argc, char **argv)
 	if (!g_option_context_parse (opt_ctx, &argc, &argv, &error)) {
 		g_warning ("%s\n", error->message);
 		g_error_free (error);
+		g_free (d);
 		return 1;
 	}
 
@@ -780,7 +786,7 @@ main (int argc, char **argv)
 	                                     G_OBJECT (d->handler));
 
 	if (!persist)
-		d->quit_timeout = g_timeout_add (10000, quit_timeout_cb, NULL);
+		d->quit_timeout = g_timeout_add_seconds (10, quit_timeout_cb, NULL);
 
 	g_main_loop_run (loop);
 
