@@ -31,12 +31,11 @@
 #include <net/ethernet.h>
 #include <netinet/ether.h>
 #include <pwd.h>
-
-#include <NetworkManager.h>
-#include <nm-connection.h>
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
+#include <NetworkManager.h>
+#include <nm-connection.h>
 #include <nm-setting-8021x.h>
 #include <nm-setting-bluetooth.h>
 #include <nm-setting-cdma.h>
@@ -66,6 +65,7 @@
 #include "nm-session-monitor.h"
 #include "plugins/keyfile/plugin.h"
 #include "nm-agent-manager.h"
+#include "nm-settings-utils.h"
 
 #define CONFIG_KEY_NO_AUTO_DEFAULT "no-auto-default"
 
@@ -1329,11 +1329,13 @@ default_wired_try_update (NMDefaultWiredConnection *wired,
 void
 nm_settings_device_added (NMSettings *self, NMDevice *device)
 {
+	NMSettingsPrivate *priv = NM_SETTINGS_GET_PRIVATE (self);
 	GByteArray *mac = NULL;
 	struct ether_addr tmp;
 	NMDefaultWiredConnection *wired;
 	gboolean read_only = TRUE;
 	const char *id;
+	char *defname;
 
 	if (nm_device_get_device_type (device) != NM_DEVICE_TYPE_ETHERNET)
 		return;
@@ -1357,7 +1359,9 @@ nm_settings_device_added (NMSettings *self, NMDevice *device)
 	if (get_plugin (self, NM_SYSTEM_CONFIG_INTERFACE_CAP_MODIFY_CONNECTIONS))
 		read_only = FALSE;
 
-	wired = nm_default_wired_connection_new (mac, device, read_only);
+	defname = nm_settings_utils_get_default_wired_name (priv->connections);
+	wired = nm_default_wired_connection_new (mac, device, defname, read_only);
+	g_free (defname);
 	if (!wired)
 		goto ignore;
 
